package controllers

import model.Person
import org.slf4j.MDC
import play.api.Logger
import play.api.libs.functional.syntax._
import play.api.libs.json._
import play.api.mvc._

import scala.collection.mutable

class PersonController extends Controller {

  val repository: mutable.MutableList[Person] = new mutable.MutableList[Person]()
  val logger: Logger = Logger(this.getClass())

  implicit val personWrites: Writes[Person] = (
      (JsPath \ "name").write[String] and
      (JsPath \ "age").write[Int]
    )(unlift(Person.unapply))
  implicit val personReads: Reads[Person] = (
      (JsPath \ "name").read[String] and
      (JsPath \ "age").read[Int]
    )(Person.apply _)

  def all = Action {
    MDC.put("method", "all")
    logger.info(s"returning ${repository.size} people")
    Ok(Json.toJson(repository)).as(JSON)
  }

  def insert = Action(BodyParsers.parse.json) { request =>
    MDC.put("method", "insert")
    logger.trace(s"received json: ${request.body.toString()}")
    request.body.validate[Person].fold(
      errors => {
        logger.error(s"invalid json: ${errors.toString()}")
        BadRequest("invalid json person")
      },
      person => {
        logger.debug(s"person to insert: ${person.toString}")
        repository.+=(person)
        logger.info("success")
        Ok
      }
    )
  }

}
